/**
 * 针对画布的辅助工具类
 * 1. 灵活调整大小与视网膜屏幕下的高清显示
 * 2. 按照云阅卷的图片拼接规则绘制图片内容
 * 
 * @param {} option 
 */
function EasyCanvas(option) {
    this.canvas = option.canvas
    this.context = this.canvas.getContext("2d")
    this.ratio = this.getPixelRatio()

    this.originSize = {
        width: 0,
        height: 0
    }
    this.styleSize = {
        width: 0,
        height: 0
    }
    this.boxSize = {}
    this.fontConfig = {
        name: option.fontName || 'Arial',
        size: option.fontSize || 60,
        color: option.fontColor || 'red'
    }

    if (option.boxWidth != undefined && boxWidth.boxWidth > 0) {
        this.boxSize.width = option.boxWidth
    } else if (option.boxHeight != undefined && option.boxHeight > 0) {
        this.boxSize.height = option.boxHeight
    }

    this.onClick = option.onClick || (function() {})

    let self = this
    this.canvas.onclick = function(event) {
        if (self.originSize.width > 0 && self.originSize.height > 0) {
            self.onClick({
                left: (event.offsetX - this.offsetLeft),
                top: (event.offsetY - this.offsetTop)
            })
        }
    }
}

EasyCanvas.prototype.changeBoxSize = function(width, height) {
    if (width != undefined) {
        this.boxSize.width = width
        this.boxSize.height = undefined
    } else if (height != undefined) {
        this.boxSize.height = height
        this.boxSize.width = undefined
    }
}

EasyCanvas.prototype.getPixelRatio = function() {
    let backingStore = this.context.backingStorePixelRatio ||
        this.context.webkitBackingStorePixelRatio ||
        this.context.mozBackingStorePixelRatio ||
        this.context.msBackingStorePixelRatio ||
        this.context.oBackingStorePixelRatio ||
        this.context.backingStorePixelRatio || 1;
    return (window.devicePixelRatio || 1) / backingStore;
}

EasyCanvas.prototype.clear = function() {
    this.context.clearRect(0, 0, this.canvas.width, this.canvas.heigth)
}

EasyCanvas.prototype.reset = function(width, height) {
    this.originSize.width = Math.max((width || 0), 0)
    this.originSize.height = Math.max((height || 0), 0)
    this.styleSize.width = this.originSize.width
    this.styleSize.height = this.originSize.height

    if (this.styleSize.width > 0 && this.styleSize.height > 0) {
        if (this.boxSize.width != undefined) {
            this.styleSize.width = Math.min(this.originSize.width, this.boxSize.width)
            this.styleSize.height = this.styleSize.width * this.originSize.height / this.originSize.width
        } else if (this.boxSize.height != undefined) {
            this.styleSize.height = Math.min(this.originSize.height, this.boxSize.height)
            this.styleSize.width = this.styleSize.height * this.originSize.width / this.originSize.height
        }
    }

    this.canvas.width = this.styleSize.width * this.ratio
    this.canvas.height = this.styleSize.height * this.ratio
    this.canvas.style.width = this.styleSize.width + 'px'
    this.canvas.style.height = this.styleSize.height + 'px'

    this.clear()
}

EasyCanvas.prototype.drawImage = function(image) {
    this.drawImages([image], undefined, 1.0)
}

EasyCanvas.prototype.drawImages = function(images, config, scale) {
    scale = scale || 1.0
    if (config == undefined || config.length == 0) {
        //不指定拼接配置时，默认按顺序显示所有图片
        config = []
        for (let i = 0; i < images.length; i++) {
            config.push({
                i: i + 1,
                x: 0,
                y: 0,
                w: 0,
                h: 0
            })
        }
    }
    let maxWidth = 0;
    let totalHeight = 0;
    for (let i = 0; i < config.length; i++) {
        //计算最大宽度与合计高度
        if (config[i].w <= 0) {
            config[i].w = images[config[i].i - 1].width;
        }
        if (config[i].h <= 0) {
            config[i].h = images[config[i].i - 1].height;
        }
        maxWidth = Math.max(maxWidth, config[i].w * scale);
        totalHeight += config[i].h * scale;
    }
    if (maxWidth > 0 && totalHeight > 0) {
        //设置画布大小
        this.reset(maxWidth, totalHeight)
        //绘画到画布
        scale = scale * this.styleSize.width / this.originSize.width
        let height = 0;
        for (let i = 0; i < config.length; i++) {
            this.context.drawImage(images[config[i].i - 1], config[i].x, config[i].y, config[i].w, config[i].h, 0, height, config[i].w * scale * this.ratio, config[i].h * scale * this.ratio);
            height += (config[i].h * scale * this.ratio);
        }
    } else {
        this.reset()
    }
}

EasyCanvas.prototype.drawText = function(text, left, top) {
    this.context.font = this.fontConfig.size * this.ratio + 'px ' + this.fontConfig.name
    this.context.fillStyle = this.fontConfig.color
    this.context.fillText(text, left * this.ratio, top * this.ratio)
}

EasyCanvas.prototype.getImageData = function(format) {
    return this.canvas.toDataUrl(format ? format : 'image/jpeg')
}